/*==============================================================================
*NOME:    SEMAF.C
*
*Projeto: Leitores e Escritores
*
*Verso:  V 1.00
*Autores: Flavio Alexandre da Rocha Magacho Filho
*         Juliana Carpes Imperial
*         Silvia Moreira
*
*Descrio do mdulo de implementao
*   Objetivos:
*      - Implementar uma funo que cria um semforo;
*      - Implementar uma funo que faz a operao P( ) em um semforo;
*      - Implementar uma funo que faz a operao V( ) em um semforo e
*      - Implementar uma funo que destroi um semforo.
*
*Modo de utilizar:
*      - A operao de criao de semforo  independente da operao de
*        atribuio de valor inicial. Isso pode criar situaes de corrida entre
*        dois processos. A criao e inicializao por um processo "pai de
*        todos" evita esse problema.
*      - Um semforo SV deve sempre ser eliminado exlicitamente pelo programa
*        que usa esta biblioteca, usando a funo que destroi semforos. Caso
*        contrrio, os recursos no so liberados pelo sistema operacional.
*      - As funes aqui implementadas tratam os casos de erro atravs da
*        varivel global errno; quando h erro, retornam -1.
*=============================================================================*/

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>


#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>


#define  semaf_OWN
#include "semaf.h"
#undef   semaf_OWN

/*==============================================================================
*Nome: sem_create
*=============================================================================*/

int sem_create( int val ) {

/* uniao para inicializar o semforo */
   union semun {
      int              val ;
      struct semid_ds* buf ;
   } ArgSem ;

   FILE*           pArq         ;
   static int      iChaveID = 1 ;  /* id da primeira chave para a criao */
   int             iFlag        ,  /* flag para a criao do semforo */
                   iSemID       ,  /* vai receber o id do semforo criado */
                   iIniciou     ;  /* resultado da inicializao do semforo */
   key_t           Chave        ;  /* chave para a criao do semforo */
   struct semid_ds Info         ;  /* estrutura para inicializar o semforo */

/* tenta abrir um arquivo para a chave do semforo, se nao for possvel
   abrir, o programa  cancelado */
   pArq = fopen( SZ_ARQ, "wb" ) ;
   if ( pArq == NULL ) {
      strerror( errno ) ;
      exit( 1 ) ;
   }
   fclose( pArq ) ;

   Chave = ftok( SZ_ARQ, iChaveID ) ; /* obtem chave p/criao do semaforo */

/* prepara flags para a criao do semforo, para que indique erro se semaforo
   j existir e permita leitura e alteracao por esse processo */
   iFlag = IPC_CREAT | IPC_EXCL | S_IRUSR | S_IWUSR ;

   iSemID = semget( Chave, 1, iFlag ) ; /* tenta criar 1 semforo */

   if ( iSemID < 0 ) {  /* se no criou, emite mensagem de erro */
      printf( "%s\n", strerror( errno ) ) ;
      return -1 ;
   } /* if */

   ArgSem.buf = &Info ;
   ArgSem.val = val ;   /* valor inicial do semaforo */

/* tenta inicializar semforo */
   iIniciou = semctl( iSemID, 0, SETVAL, ArgSem ) ;

/* se no inicializou, emite mensagem de erro e destri o semforo */
   if ( iIniciou < 0 ) {

      int iDestruiu ;  /* resultado da destruio do semforo */
      printf( "%s\n", strerror( errno ) ) ;

   /* tenta destruir semforo */
      iDestruiu = semctl( iSemID, 0, IPC_RMID, 1 ) ;

   /* se no puder destruir, emite mensagem de erro e incrementa iChaveID */
      if ( iDestruiu < 0 ) {
         printf( "%s\n", strerror( errno ) ) ;

      /* aumenta id da chave para prxima chamada se sem_create, j que semforo
         no pde ser destrudo */
         iChaveID++ ;
      } /* if */

      return -1 ;

   } else {        /* pde criar e inicializar semforo */
      iChaveID++ ; /* aumenta id da chave para prxima chamada se sem_create */
      return iSemID ;
   } /* if */

} /* sem_create */


/*==============================================================================
*Nome: sem_p
*=============================================================================*/

int sem_p( int semid ) {
   struct sembuf SemBuf ;    /* estrutura para operar o semforo */
   int           iFezP  ;    /* resultado da operao P( ) do semforo */

/* inicializa estrutura para a operao P( ) */
   SemBuf.sem_num = 0 ;
   SemBuf.sem_op = -1 ;
   SemBuf.sem_flg = 0 ;

   iFezP = semop( semid, &SemBuf, 1 ) ;     /* tenta fazer P( ) */

   if ( iFezP < 0 ) {
      printf( "%s\n", strerror( errno ) ) ;
      return -1 ;                           /* no conseguiu fazer P( ) */
   } /* if */

   return 0 ;                               /* fez P( ) com sucesso */
} /* sem_p */


/*==============================================================================
*Nome: sem_v
*=============================================================================*/

int sem_v( int semid ) {
   struct sembuf SemBuf ;    /* estrutura para operar o semforo */
   int           iFezV  ;    /* resultado da operao V( ) do semforo */

/* inicializa estrutura para a operao V( ) */
   SemBuf.sem_num = 0 ;
   SemBuf.sem_op = 1 ;
   SemBuf.sem_flg = 0 ;

   iFezV = semop( semid, &SemBuf, 1 ) ;     /* tenta fazer V( ) */

   if ( iFezV < 0 ) {
      printf( "%s\n", strerror( errno ) ) ;
      return -1 ;                           /* no conseguiu fazer V( ) */
   } /* if */

   return 0 ;                               /* fez V( ) com sucesso */
} /* sem_v */


/*==============================================================================
*Nome: sem_destroy
*=============================================================================*/

int sem_destroy( int semid ) {

   int iDestruiu = semctl( semid, 0, IPC_RMID, 1 ) ;

   if ( iDestruiu < 0 ) {
      printf( "%s\n", strerror( errno ) ) ;
      return -1 ;                           /* no pde destruir o semforo */
   } /* if */

   return 0 ;                               /* semforo destrudo com sucesso */

} /* sem_destroy */


/* semaf.c */
