/*==============================================================================
*NOME:    SEMAF.H
*
*Projeto: Leitores e Escritores
*
*Verso:  V 1.00
*Autores: Flvio Alexandre da Rocha Magacho Filho
*         Juliana Carpes Imperial
*         Silvia Moreira
*
*Descrio do mdulo de declarao
*   Objetivos:
*      - Definir uma funo que cria um semforo;
*      - Definir uma funo que faz a operao P( ) em um semforo;
*      - Definir uma funo que faz a operao V( ) em um semforo e
*      - Definir uma funo que destroi um semforo.
*
*Modo de utilizar:
*      - A operao de criao de semforo  independente da operao de
*        atribuio de valor inicial. Isso pode criar situaes de corrida entre
*        dois processos. A criao e inicializao por um processo "pai de
*        todos" evita esse problema.
*      - Um semforo SV deve sempre ser eliminado exlicitamente pelo programa
*        que usa esta biblioteca, usando a funo que destroi semforos. Caso
*        contrrio, os recursos no so liberados pelo sistema operacional.
*=============================================================================*/


#if !defined( semaf_MOD )
#define       semaf_MOD

#if defined( semaf_OWN )
	#define   semaf_EXT
#else
	#define   semaf_EXT extern
#endif


#define SZ_ARQ "arqtemp"


/*==============================================================================
*Nome: sem_create
*Descrio da Funo: funo que cria um semforo.
*
*Parametro recebido
*   val: o valor inicial do semforo.
*
*Valor Retornado
*   Retorna um identificador do novo semforo, caso no seja possvel criar o
*   novo semforo, a funo retorna -1.
*=============================================================================*/

int sem_create( int val ) ;


/*==============================================================================
*Nome: sem_p
*Descrio da Funo: esta funo decrementa o valor de um semforo, se esse
*                     valor for positivo, ou deixa o processo que a chama
*                     bloqueado na fila desse semforo em caso contrrio.
*
*Parametro recebido
*   semid: identificador de semforo.
*
*Valor Retornado
*   Caso no seja possvel realizar a operao P( ), retorna -1.
*   Em caso de sucesso, retorna 0.
*=============================================================================*/

int sem_p( int semid ) ;


/*==============================================================================
*Nome: sem_v
*Descrio da Funo: esta funo desbloqueia o primeiro processo da fila de um
*                     semforo, se houver algum processo nessa fila, ou
*                     incrementa o valor do semforo em caso contrrio.
*
*Parametro recebido
*   semid: identificador de semforo.
*
*Valor Retornado
*   Caso no seja possvel realizar a operao V( ), retorna -1.
*   Em caso de sucesso, retorna 0.
*=============================================================================*/

int sem_v( int semid ) ;


/*==============================================================================
*Nome: sem_destroy
*Descrio da Funo: esta funo destri um semforo, liberando os recursos
*                     alocados para ele no sistema operacional
*
*Parametro recebido
*   semid: identificador de semforo.
*
*Valor Retornado
*   Caso no seja possvel destruir o semforo, retorna -1.
*   Em caso de sucesso, retorna 0.
*=============================================================================*/

int sem_destroy( int semid ) ;


#undef semaf_EXT

#endif


/* semaf.h */